package javax.imageio;

import java.awt.image.MultiPixelPackedSampleModel;
import java.awt.image.BandedSampleModel;
import java.awt.image.PixelInterleavedSampleModel;
import java.awt.image.DirectColorModel;
import java.awt.image.IndexColorModel;
import java.util.Hashtable;
import java.awt.image.Raster;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.awt.image.ComponentColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.RenderedImage;
import java.awt.color.ColorSpace;
import java.awt.image.SampleModel;
import java.awt.image.ColorModel;

public class ImageTypeSpecifier
{
    protected ColorModel colorModel;
    protected SampleModel sampleModel;
    private static ImageTypeSpecifier[] BISpecifier;
    private static ColorSpace sRGB;
    
    private ImageTypeSpecifier() {
    }
    
    public ImageTypeSpecifier(ColorModel colorModel, SampleModel sampleModel) {
        if (colorModel == null) {
            throw new IllegalArgumentException("colorModel == null!");
        }
        if (sampleModel == null) {
            throw new IllegalArgumentException("sampleModel == null!");
        }
        if (!colorModel.isCompatibleSampleModel(sampleModel)) {
            throw new IllegalArgumentException("sampleModel is incompatible with colorModel!");
        }
        this.colorModel = colorModel;
        this.sampleModel = sampleModel;
        return;
    }
    
    public ImageTypeSpecifier(RenderedImage renderedImage) {
        if (renderedImage == null) {
            throw new IllegalArgumentException("image == null!");
        }
        this.colorModel = renderedImage.getColorModel();
        this.sampleModel = renderedImage.getSampleModel();
        return;
    }
    
    public static ImageTypeSpecifier createPacked(ColorSpace colorSpace, int n, int n2, int n3, int n4, int n5, boolean b) {
        return new ImageTypeSpecifier.Packed(colorSpace, n, n2, n3, n4, n5, b);
    }
    
    static ColorModel createComponentCM(ColorSpace colorSpace, int n, int n2, boolean hasAlpha, boolean isAlphaPremultiplied) {
        int transparency = hasAlpha ? 3 : 1;
        int[] bits = new int[n];
        int dataTypeSize = DataBuffer.getDataTypeSize(n2);
        for (int i = 0; i < n; ++i) {
            bits[i] = dataTypeSize;
        }
        return new ComponentColorModel(colorSpace, bits, hasAlpha, isAlphaPremultiplied, transparency, n2);
    }
    
    public static ImageTypeSpecifier createInterleaved(ColorSpace colorSpace, int[] array, int n, boolean b, boolean b2) {
        return new ImageTypeSpecifier.Interleaved(colorSpace, array, n, b, b2);
    }
    
    public static ImageTypeSpecifier createBanded(ColorSpace colorSpace, int[] array, int[] array2, int n, boolean b, boolean b2) {
        return new ImageTypeSpecifier.Banded(colorSpace, array, array2, n, b, b2);
    }
    
    public static ImageTypeSpecifier createGrayscale(int n, int n2, boolean b) {
        return new ImageTypeSpecifier.Grayscale(n, n2, b, false, false);
    }
    
    public static ImageTypeSpecifier createGrayscale(int n, int n2, boolean b, boolean b2) {
        return new ImageTypeSpecifier.Grayscale(n, n2, b, true, b2);
    }
    
    public static ImageTypeSpecifier createIndexed(byte[] array, byte[] array2, byte[] array3, byte[] array4, int n, int n2) {
        return new ImageTypeSpecifier.Indexed(array, array2, array3, array4, n, n2);
    }
    
    public static ImageTypeSpecifier createFromBufferedImageType(int n) {
        if (n >= 1 && n <= 13) {
            return getSpecifier(n);
        }
        if (n == 0) {
            throw new IllegalArgumentException("Cannot create from TYPE_CUSTOM!");
        }
        throw new IllegalArgumentException("Invalid BufferedImage type!");
    }
    
    public static ImageTypeSpecifier createFromRenderedImage(RenderedImage renderedImage) {
        if (renderedImage == null) {
            throw new IllegalArgumentException("image == null!");
        }
        if ((renderedImage instanceof BufferedImage)) {
            int type = ((BufferedImage)renderedImage).getType();
            if (type != 0) {
                return getSpecifier(type);
            }
        }
        return new ImageTypeSpecifier(renderedImage);
    }
    
    public int getBufferedImageType() {
        return this.createBufferedImage(1, 1).getType();
    }
    
    public int getNumComponents() {
        return this.colorModel.getNumComponents();
    }
    
    public int getNumBands() {
        return this.sampleModel.getNumBands();
    }
    
    public int getBitsPerBand(int n) {
        if ((n < 0 | n >= this.getNumBands())) {
            throw new IllegalArgumentException("band out of range!");
        }
        return this.sampleModel.getSampleSize(n);
    }
    
    public SampleModel getSampleModel() {
        return this.sampleModel;
    }
    
    public SampleModel getSampleModel(int n, int n2) {
        if ((long)n * (long)n2 > 2147483647L) {
            throw new IllegalArgumentException("width*height > Integer.MAX_VALUE!");
        }
        return this.sampleModel.createCompatibleSampleModel(n, n2);
    }
    
    public ColorModel getColorModel() {
        return this.colorModel;
    }
    
    public BufferedImage createBufferedImage(int n, int n2) {
        try {
            return new BufferedImage(this.colorModel, Raster.createWritableRaster(this.getSampleModel(n, n2), new Point(0, 0)), this.colorModel.isAlphaPremultiplied(), new Hashtable());
        }
        catch (NegativeArraySizeException ex) {
            throw new IllegalArgumentException("Array size > Integer.MAX_VALUE!");
        }
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == null || !(o instanceof ImageTypeSpecifier)) {
            return false;
        }
        ImageTypeSpecifier imageTypeSpecifier = (ImageTypeSpecifier)o;
        return this.colorModel.equals(imageTypeSpecifier.colorModel) && this.sampleModel.equals(imageTypeSpecifier.sampleModel);
    }
    
    @Override
    public int hashCode() {
        return 9 * this.colorModel.hashCode() + 14 * this.sampleModel.hashCode();
    }
    
    private static ImageTypeSpecifier getSpecifier(int n) {
        if (ImageTypeSpecifier.BISpecifier[n] == null) {
            ImageTypeSpecifier.BISpecifier[n] = createSpecifier(n);
        }
        return ImageTypeSpecifier.BISpecifier[n];
    }
    
    private static ImageTypeSpecifier createSpecifier(int n) {
        switch (n) {
            case 1: {
                return createPacked(ImageTypeSpecifier.sRGB, 16711680, 65280, 255, 0, 3, false);
            }
            case 2: {
                return createPacked(ImageTypeSpecifier.sRGB, 16711680, 65280, 255, -16777216, 3, false);
            }
            case 3: {
                return createPacked(ImageTypeSpecifier.sRGB, 16711680, 65280, 255, -16777216, 3, true);
            }
            case 4: {
                return createPacked(ImageTypeSpecifier.sRGB, 255, 65280, 16711680, 0, 3, false);
            }
            case 5: {
                return createInterleaved(ImageTypeSpecifier.sRGB, new int[] { 2, 1, 0 }, 0, false, false);
            }
            case 6: {
                return createInterleaved(ImageTypeSpecifier.sRGB, new int[] { 3, 2, 1, 0 }, 0, true, false);
            }
            case 7: {
                return createInterleaved(ImageTypeSpecifier.sRGB, new int[] { 3, 2, 1, 0 }, 0, true, true);
            }
            case 8: {
                return createPacked(ImageTypeSpecifier.sRGB, 63488, 2016, 31, 0, 1, false);
            }
            case 9: {
                return createPacked(ImageTypeSpecifier.sRGB, 31744, 992, 31, 0, 1, false);
            }
            case 10: {
                return createGrayscale(8, 0, false);
            }
            case 11: {
                return createGrayscale(16, 1, false);
            }
            case 12: {
                return createGrayscale(1, 0, false);
            }
            case 13: {
                IndexColorModel indexColorModel = (IndexColorModel)new BufferedImage(1, 1, 13).getColorModel();
                int mapSize = indexColorModel.getMapSize();
                byte[] r = new byte[mapSize];
                byte[] g = new byte[mapSize];
                byte[] b = new byte[mapSize];
                byte[] a = new byte[mapSize];
                indexColorModel.getReds(r);
                indexColorModel.getGreens(g);
                indexColorModel.getBlues(b);
                indexColorModel.getAlphas(a);
                return createIndexed(r, g, b, a, 8, 0);
            }
            default: {
                throw new IllegalArgumentException("Invalid BufferedImage type!");
            }
        }
    }
    
    ImageTypeSpecifier(ImageTypeSpecifier.1 object) {
        this();
    }
    
    static {
        ImageTypeSpecifier.sRGB = ColorSpace.getInstance(1000);
        ImageTypeSpecifier.BISpecifier = new ImageTypeSpecifier[14];
    }
    
    static class Packed extends ImageTypeSpecifier
    {
        ColorSpace colorSpace;
        int redMask;
        int greenMask;
        int blueMask;
        int alphaMask;
        int transferType;
        boolean isAlphaPremultiplied;
        
        public Packed(ColorSpace colorSpace, int n, int n2, int n3, int n4, int n5, boolean b) {
            super(null);
            if (colorSpace == null) {
                throw new IllegalArgumentException("colorSpace == null!");
            }
            if (colorSpace.getType() != 5) {
                throw new IllegalArgumentException("colorSpace is not of type TYPE_RGB!");
            }
            if (n5 != 0 && n5 != 1 && n5 != 3) {
                throw new IllegalArgumentException("Bad value for transferType!");
            }
            if (n == 0 && n2 == 0 && n3 == 0 && n4 == 0) {
                throw new IllegalArgumentException("No mask has at least 1 bit set!");
            }
            this.colorSpace = colorSpace;
            this.redMask = n;
            this.greenMask = n2;
            this.blueMask = n3;
            this.alphaMask = n4;
            this.transferType = n5;
            this.isAlphaPremultiplied = b;
            this.colorModel = new DirectColorModel(colorSpace, 32, n, n2, n3, n4, b, n5);
            this.sampleModel = this.colorModel.createCompatibleSampleModel(1, 1);
            return;
        }
    }
    
    static class Interleaved extends ImageTypeSpecifier
    {
        ColorSpace colorSpace;
        int[] bandOffsets;
        int dataType;
        boolean hasAlpha;
        boolean isAlphaPremultiplied;
        
        public Interleaved(ColorSpace colorSpace, int[] bandOffsets, int n, boolean hasAlpha, boolean isAlphaPremultiplied) {
            super(null);
            if (colorSpace == null) {
                throw new IllegalArgumentException("colorSpace == null!");
            }
            if (bandOffsets == null) {
                throw new IllegalArgumentException("bandOffsets == null!");
            }
            if (bandOffsets.length != colorSpace.getNumComponents() + (hasAlpha ? 1 : 0)) {
                throw new IllegalArgumentException("bandOffsets.length is wrong!");
            }
            if (n != 0 && n != 2 && n != 1 && n != 3 && n != 4 && n != 5) {
                throw new IllegalArgumentException("Bad value for dataType!");
            }
            this.colorSpace = colorSpace;
            this.bandOffsets = (int[])(int[])bandOffsets.clone();
            this.dataType = n;
            this.hasAlpha = hasAlpha;
            this.isAlphaPremultiplied = isAlphaPremultiplied;
            this.colorModel = ImageTypeSpecifier.createComponentCM(colorSpace, bandOffsets.length, n, hasAlpha, isAlphaPremultiplied);
            int n2 = bandOffsets[0];
            for (int i = 0; i < bandOffsets.length; ++i) {
                int n3 = bandOffsets[i];
                n2 = Math.min(n3, n2);
                n2 = Math.max(n3, n2);
            }
            int pixelStride = n2 - n2 + 1;
            int w = 1;
            this.sampleModel = new PixelInterleavedSampleModel(n, w, 1, pixelStride, w * pixelStride, bandOffsets);
            return;
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == null || !(o instanceof ImageTypeSpecifier.Interleaved)) {
                return false;
            }
            ImageTypeSpecifier.Interleaved interleaved = (ImageTypeSpecifier.Interleaved)o;
            if (!this.colorSpace.equals(interleaved.colorSpace) || this.dataType != interleaved.dataType || this.hasAlpha != interleaved.hasAlpha || this.isAlphaPremultiplied != interleaved.isAlphaPremultiplied || this.bandOffsets.length != interleaved.bandOffsets.length) {
                return false;
            }
            int i = 0;
            while (i < this.bandOffsets.length) {
                if (this.bandOffsets[i] != interleaved.bandOffsets[i]) {
                    return false;
                }
                ++i;
            }
            return true;
        }
        
        @Override
        public int hashCode() {
            return super.hashCode() + 4 * this.bandOffsets.length + 25 * this.dataType + (this.hasAlpha ? 17 : 18);
        }
    }
    
    static class Banded extends ImageTypeSpecifier
    {
        ColorSpace colorSpace;
        int[] bankIndices;
        int[] bandOffsets;
        int dataType;
        boolean hasAlpha;
        boolean isAlphaPremultiplied;
        
        public Banded(ColorSpace colorSpace, int[] bankIndices, int[] bandOffsets, int n, boolean hasAlpha, boolean isAlphaPremultiplied) {
            super(null);
            if (colorSpace == null) {
                throw new IllegalArgumentException("colorSpace == null!");
            }
            if (bankIndices == null) {
                throw new IllegalArgumentException("bankIndices == null!");
            }
            if (bandOffsets == null) {
                throw new IllegalArgumentException("bandOffsets == null!");
            }
            if (bankIndices.length != bandOffsets.length) {
                throw new IllegalArgumentException("bankIndices.length != bandOffsets.length!");
            }
            if (n != 0 && n != 2 && n != 1 && n != 3 && n != 4 && n != 5) {
                throw new IllegalArgumentException("Bad value for dataType!");
            }
            if (bandOffsets.length != colorSpace.getNumComponents() + (hasAlpha ? 1 : 0)) {
                throw new IllegalArgumentException("bandOffsets.length is wrong!");
            }
            this.colorSpace = colorSpace;
            this.bankIndices = (int[])(int[])bankIndices.clone();
            this.bandOffsets = (int[])(int[])bandOffsets.clone();
            this.dataType = n;
            this.hasAlpha = hasAlpha;
            this.isAlphaPremultiplied = isAlphaPremultiplied;
            this.colorModel = ImageTypeSpecifier.createComponentCM(colorSpace, bankIndices.length, n, hasAlpha, isAlphaPremultiplied);
            int n2 = 1;
            this.sampleModel = new BandedSampleModel(n, n2, 1, n2, bankIndices, bandOffsets);
            return;
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == null || !(o instanceof ImageTypeSpecifier.Banded)) {
                return false;
            }
            ImageTypeSpecifier.Banded banded = (ImageTypeSpecifier.Banded)o;
            if (!this.colorSpace.equals(banded.colorSpace) || this.dataType != banded.dataType || this.hasAlpha != banded.hasAlpha || this.isAlphaPremultiplied != banded.isAlphaPremultiplied || this.bankIndices.length != banded.bankIndices.length || this.bandOffsets.length != banded.bandOffsets.length) {
                return false;
            }
            int i = 0;
            while (i < this.bankIndices.length) {
                if (this.bankIndices[i] != banded.bankIndices[i]) {
                    return false;
                }
                ++i;
            }
            int j = 0;
            while (j < this.bandOffsets.length) {
                if (this.bandOffsets[j] != banded.bandOffsets[j]) {
                    return false;
                }
                ++j;
            }
            return true;
        }
        
        @Override
        public int hashCode() {
            return super.hashCode() + 3 * this.bandOffsets.length + 7 * this.bankIndices.length + 21 * this.dataType + (this.hasAlpha ? 19 : 29);
        }
    }
    
    static class Grayscale extends ImageTypeSpecifier
    {
        int bits;
        int dataType;
        boolean isSigned;
        boolean hasAlpha;
        boolean isAlphaPremultiplied;
        
        public Grayscale(int numberOfBits, int n, boolean isSigned, boolean b, boolean b2) {
            super(null);
            if (numberOfBits != 1 && numberOfBits != 2 && numberOfBits != 4 && numberOfBits != 8 && numberOfBits != 16) {
                throw new IllegalArgumentException("Bad value for bits!");
            }
            if (n != 0 && n != 2 && n != 1) {
                throw new IllegalArgumentException("Bad value for dataType!");
            }
            if (numberOfBits > 8 && n == 0) {
                throw new IllegalArgumentException("Too many bits for dataType!");
            }
            this.bits = numberOfBits;
            this.dataType = n;
            this.isSigned = isSigned;
            this.hasAlpha = b;
            this.isAlphaPremultiplied = b2;
            ColorSpace instance = ColorSpace.getInstance(1003);
            if ((numberOfBits == 8 && n == 0) || (numberOfBits == 16 && (n == 2 || n == 1))) {
                int pixelStride = b ? 2 : 1;
                int transparency = b ? 3 : 1;
                int[] bits = new int[pixelStride];
                bits[0] = numberOfBits;
                if (pixelStride == 2) {
                    bits[1] = numberOfBits;
                }
                this.colorModel = new ComponentColorModel(instance, bits, b, b2, transparency, n);
                int[] bandOffsets = new int[pixelStride];
                bandOffsets[0] = 0;
                if (pixelStride == 2) {
                    bandOffsets[1] = 1;
                }
                int w = 1;
                this.sampleModel = new PixelInterleavedSampleModel(n, w, 1, pixelStride, w * pixelStride, bandOffsets);
            }
            else {
                int size = 1 << numberOfBits;
                byte[] array = new byte[size];
                for (int i = 0; i < size; ++i) {
                    array[i] = (byte)(i * 255 / (size - 1));
                }
                this.colorModel = new IndexColorModel(numberOfBits, size, array, array, array);
                this.sampleModel = new MultiPixelPackedSampleModel(n, 1, 1, numberOfBits);
            }
            return;
        }
    }
    
    static class Indexed extends ImageTypeSpecifier
    {
        byte[] redLUT;
        byte[] greenLUT;
        byte[] blueLUT;
        byte[] alphaLUT;
        int bits;
        int dataType;
        
        public Indexed(byte[] array, byte[] array2, byte[] array3, byte[] a, int n, int dataType) {
            super(null);
            this.alphaLUT = null;
            if (array == null || array2 == null || array3 == null) {
                throw new IllegalArgumentException("LUT is null!");
            }
            if (n != 1 && n != 2 && n != 4 && n != 8 && n != 16) {
                throw new IllegalArgumentException("Bad value for bits!");
            }
            if (dataType != 0 && dataType != 2 && dataType != 1 && dataType != 3) {
                throw new IllegalArgumentException("Bad value for dataType!");
            }
            if ((n > 8 && dataType == 0) || (n > 16 && dataType != 3)) {
                throw new IllegalArgumentException("Too many bits for dataType!");
            }
            int n2 = 1 << n;
            if (array.length != n2 || array2.length != n2 || array3.length != n2 || (a != null && a.length != n2)) {
                throw new IllegalArgumentException("LUT has improper length!");
            }
            this.redLUT = (byte[])(byte[])array.clone();
            this.greenLUT = (byte[])(byte[])array2.clone();
            this.blueLUT = (byte[])(byte[])array3.clone();
            if (a != null) {
                this.alphaLUT = (byte[])(byte[])a.clone();
            }
            this.bits = n;
            this.dataType = dataType;
            if (a == null) {
                this.colorModel = new IndexColorModel(n, array.length, array, array2, array3);
            }
            else {
                this.colorModel = new IndexColorModel(n, array.length, array, array2, array3, a);
            }
            if ((n == 8 && dataType == 0) || (n == 16 && (dataType == 2 || dataType == 1))) {
                this.sampleModel = new PixelInterleavedSampleModel(dataType, 1, 1, 1, 1, new int[] { 0 });
            }
            else {
                this.sampleModel = new MultiPixelPackedSampleModel(dataType, 1, 1, n);
            }
            return;
        }
    }
}
